/*
 * Copyright (C) 2007 - John Anderson, Neil Kettle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#ifdef DEBUG
#include <assert.h>
#endif
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>

#include "common.h"
#include "net.h"
#include "synscan.h"

void
fatal (const char *format, ...)
{
  va_list ap;

  va_start (ap, format);

  if (format)
    vfprintf (stderr, format, ap);

  va_end (ap);
  fflush (stderr);

  exit (EXIT_FAILURE);
}

void
warning (const char *format, ...)
{
  va_list ap;

  va_start (ap, format);

  if (format)
    vfprintf (stderr, format, ap);

  va_end (ap);
  fflush (stderr);
}

char *
rlookup (unsigned long ip)
{
  static char hostname[HOSTNAME_MAX];
  char *host;
  struct hostent *hostp;
  struct sockaddr_in sock_addr;

  sock_addr.sin_addr.s_addr = ip;
  hostp =
    gethostbyaddr ((char *) &sock_addr.sin_addr, sizeof sock_addr.sin_addr,
                   AF_INET);

  if (hostp == NULL)
    {
      struct in_addr insock_addr;

      insock_addr.s_addr = ip;
      host = inet_ntoa (insock_addr);
    }
  else
    host = hostp->h_name;

  strncpy (hostname, host, sizeof hostname - 1);
  hostname[sizeof hostname - 1] = '\0';

  return (hostname);
}

char *
nlookup (unsigned long ip)
{
  static char hostname[HOSTNAME_MAX];
  struct in_addr insock_addr;

  insock_addr.s_addr = ip;
  strncpy (hostname, inet_ntoa (insock_addr), sizeof hostname - 1);
  hostname[sizeof hostname - 1] = '\0';

  return (hostname);
}

unsigned long int
lookup (char *hostname)
{
  struct hostent *hostp;
  unsigned long int address;

  if ((address = inet_addr (hostname)) != -1)
    return (address);

  if ((hostp = gethostbyname (hostname)) == NULL)
    return -1;

#ifdef DEBUG
  assert (hostp->h_length <= sizeof address);
#endif
  memcpy (&address, hostp->h_addr, hostp->h_length);

  return (address);
}
